

function [rf gsar lsar pp deltat system opt spokes sar niters time f_final] = Spokes_SARPowerConst_deltatOpt_fmincon_v9(filepath,display,do_mls,deltat_max,B0_slice_grad,TE,freqs,freq_weights,target_phase,duty_cycle)
% function [rf gsar lsar pp deltat system opt spokes sar niters time f_final] = Spokes_SARPowerConst_deltatOpt_fmincon_v9(filepath,display,do_mls,deltat_max,B0_slice_grad,TE,freqs,freq_weights,target_phase,duty_cycle)

close all;

% read parameters
fprintf('Reading pulse parameters ...\n');
[system adj spokes opt sar]=read_spokes_def_file(filepath);
if display==1
    opt.display='on';
else
    opt.display='off';
end
spokes.deltat_max=deltat_max;

% Initial pulse & B1 maps
fprintf('Initial pulse and B1+ maps ...\n');
deltat=zeros(system.ncoils*spokes.nspokes,1);
spokes=prepare_pulse(spokes,system,opt.display,deltat,deltat_max);

% preparing optimization
fprintf('Preparing optimization ...\n');
opt=prepare_optimization(adj,opt,spokes,system,target_phase);

opt.B0_slice_grad=-B0_slice_grad;  % use the opposite of B0

if TE>(spokes.ntimes*system.deltat)
    opt.TE=TE;
elseif TE<0
    opt.TE=spokes.ntimes*system.deltat;
else
    error(sprintf('TE too short, please increase it [TE=%f  Tpulse=%f]',TE,spokes.ntimes*system.deltat));
end

% read sar matrices
fprintf('Reading SAR matrices ...\n');
sar = prepare_sar(sar,system);
sar.lsarmax = sar.lsarmax / duty_cycle;
sar.gsarmax = sar.gsarmax / duty_cycle;

% target phase profile and initial pulse
fprintf('Setting initial RF pulse and target phase profiles ...\n');
xrf0=initialize_pulse('aaa',spokes.nspokes,system.ncoils);

% system matrix
opt.freqs=freqs;
opt.freq_weights=freq_weights;
opt.nfreqs=size(freqs,2);
A_noRF=[];
A_noRF_noTS=[];
for i=1:opt.nfreqs
    [a b c d]=compute_system_matrix_deltat(opt,system,spokes,deltat,freqs(i));  % it is useful to store sin_a and sin_b 
                                                                                % for optimization of time shifts @ constant RF.
    A_noRF=[A_noRF;a];
    if i==1
        sinc_a=b;
        sinc_b=c;
    end
    A_noRF_noTS=[A_noRF_noTS;d];        
end
opt.mtt=repmat(opt.mtt,[opt.nfreqs 1]);

sinc_b_large=[];
for i=1:spokes.nspokes
    sinc_b_large=[ sinc_b_large repmat(sinc_b(:,i),[1 system.ncoils]) ];
end
sinc_b_large=repmat(sinc_b_large,[opt.nfreqs 1]);


% % BC mode
% % pha=[1:system.ncoils]'/system.ncoils*2*pi * 1;  % BC
% pha=angle(squeeze(adj.b1maps(:,end/2,end/2)));  % pseudo-CP
% rf=exp(1j*pha);
% MT=A_noRF*rf;
% voltage=mean(abs(opt.mtt))/mean(abs(MT));
% if voltage>system.vmax
%     voltage=system.vmax;
% end
% rf=rf * voltage;
% MT=A_noRF*rf;
% xrf=[ real(rf);imag(rf) ];

% JOINT OPTIMIZATION OF TIME-SHIFTS AND RF WEIGHTS
[deltat xrf f_final niters time]=optimize_RF_and_time_shifts(deltat,deltat_max,xrf0,A_noRF_noTS,sinc_a,sinc_b_large,spokes,opt,system,do_mls,sar,1e-5,0); 

spokes=time_shift_sinc_profiles(spokes,deltat,system);
if strcmpi(opt.display,'on')
    for i=1:spokes.nspokes
        figure;
        ind=[1:system.ncoils]' + (i-1)*system.ncoils;
        bar(deltat(ind)*1000); xlabel('Channel index');
        title(sprintf('Time-shifts (spokes #%d) [ms]',i));
    end
end

% display results
ncunk=system.ncoils*spokes.nspokes;
rf=xrf(1:ncunk,1) + 1j*xrf(ncunk+1:2*ncunk,1);
rf2=write_and_display_pulse(rf,opt,spokes,system);
[gsar lsar pp]=display_optimization_summary(rf,rf2,opt,spokes,sar,system,duty_cycle);

opt.B0_slice_grad=-B0_slice_grad;  % use the opposite of B0

% Bloch sim
lz=spokes.sthick;
[opt_3D adj_3D]=extend_maps_z_direction(opt,adj,20,lz,spokes);  % 20 points across the slice profile is enough to get a good estimate of the slice-average

% slice profiles
fprintf('Computing slice profiles ...\n');
% zs=([1:20]'-0.5)/20*lz - lz/2;
zs=([1:100]'-0.5)/100*lz*2 - lz;
SPs=zeros(size(zs,1),system.ncoils);
for i=1:system.ncoils
    [mz_x mz_y mz_z]=bloch_simulation_z(spokes,system,zs,i);
    SPs(:,i)=sqrt( abs(mz_x).^2 + abs(mz_y).^2 );
    SPs(:,i)=SPs(:,i) / max(SPs(:,i));
end
if strcmpi(opt.display,'on')
    figure;
    plot(zs*1e3,SPs,'-o');
    xlabel('z [mm]');
    title('Slice profiles for all channels');
    axis([min(zs)*1000 max(zs)*1000 0 1.1]);
end


% TEs=[4 8 12 16 20 24 26 30]'*1e-3;  % this is useful if one wants to simulate several TEs
TEs=opt.TE;
save_matlab_file='save ''pulse.mat'' ''rf'' ''rf2'' ''deltat'' ''times'' ''grad'' ''SPs''';
MT_SA_STA=zeros(adj.nx,adj.ny);
for i=1:size(TEs,1)
    opt_3D.TE=TEs(i);
    opt.TE=TEs(i);
    
    % SA-STA
    A = compute_system_matrix_deltat(opt,system,spokes,deltat,0);
    MT_STA(opt.ind) = A*rf;
    filename=sprintf( 'MT_SA_STA_%d',int32(1000*TEs(i)) );
    eval(sprintf('%s=MT_SA_STA;',filename));
    write_and_display_magnetization(MT_STA(opt.ind),opt);
    save_matlab_file=[save_matlab_file '  ' '''' filename ''''];
    pause(0.5);    

    % BS
    [mx my mz]=bloch_simulation(rf2,opt_3D,adj_3D,spokes,system);
    MT_Bloch=mx+1j*my;
    % MT_Bloch=MT_Bloch ./ repmat(reshape(SPs(:,1),1,1,20),[opt_3D.nx opt_3D.ny 1]);  % correction for slice-profile
    filename=sprintf( 'MT_Bloch_%d',int32(1000*TEs(i)) );
    eval(sprintf('%s=MT_Bloch;',filename));
    display_bloch_sim(MT_Bloch,opt);
    save_matlab_file=[save_matlab_file '  ' '''' filename ''''];
    pause(0.5);

end

times=[1:spokes.ntimes]'*system.deltat;
grad=spokes.g;
eval([save_matlab_file ';']);






function display_bloch_sim(mT,opt)

mtimg_av=mean( mT,3 );  % average Bloch sim over the slice thickness

faimg_av=real( asin(abs(mtimg_av))/pi*180.0 );
phaimg_av=angle(mtimg_av);

if strcmpi(opt.display,'on')
    figure;
    subplot(2,1,1); imagesc(faimg_av); colormap(hot); caxis([0 opt.tfa*1.5]); colorbar; axis image; axis off; title('Av. flip angle [deg.], Bloch sim.');
    subplot(2,1,2); imagesc(phaimg_av); colormap(hot); caxis([-pi pi]); axis image; axis off; colorbar; title('Av. phase [rad.], Bloch sim.');

    % figure;
    % ind=find(faimg_av>0);
    % [indx indy]=ind2sub( [size(mT,1) size(mT,2)],ind );
    % y_fa=zeros( size(mT,3),size(indx,1) );
    % y_pha=zeros( size(mT,3),size(indx,1) );
    % n=0;
    % for i=1:size(indx,1)
    %     n=n+1;
    %     if mod(i,100)==0
    %         y_fa(:,n)=squeeze( faimg( indx(i),indy(i),: ) );
    %         y_pha(:,n)=squeeze( phaimg( indx(i),indy(i),: ) );
    %     end
    % end
    % z=([ 1:size(mT,3) ]-0.5)'/size(mT,3)*lz - lz/2;
    % subplot(2,1,1); plot(z*100,y_fa(:,1:n)); box off; title('FA variations across slice [deg.]'); xlabel('Position within slice [cm]');
    % subplot(2,1,2); plot(z*100,y_pha(:,1:n)); box off; title('Phase variations across slice [rad.]'); xlabel('Position within slice [cm]');
end


function [opt2 adj2]=extend_maps_z_direction(opt,adj,nz,lz,spokes)

gamma=42.576*1e6;

opt2=opt;
adj2=adj;

adj2.nz=nz;
adj2.lz=lz;
adj2.zs=([1:nz]'-0.5)/nz*lz - lz/2;

% extend B1+ maps and ROI
adj2.b1maps=repmat( adj2.b1maps,[1 1 1 adj2.nz] );
adj2.roi=repmat( adj2.roi,[1 1 adj2.nz] );

% non-zero indices
opt2.ind=find(adj2.roi>0);
opt2.nnonzeropixels=size(opt2.ind,1);
[indx indy indz]=ind2sub( [adj2.nx adj2.ny adj2.nz],opt2.ind );
[tmpx tmpy tmpz]=ndgrid(adj2.xs,adj2.ys,adj2.zs);
opt2.nonzeropixels=[indx indy indz tmpx(opt2.ind) tmpy(opt2.ind) tmpz(opt2.ind)];

% create the nonzeropixels structure & B0
opt2.coords_SS=cell(spokes.nspokes,1);
g=opt.B0_slice_grad(opt.ind)/gamma;
b0=opt.B0_/gamma;
Z0=zeros(opt.nnonzeropixels,spokes.nspokes);  % pixel-dependent slice offset (also spoke-dependent)
Zeff=zeros(opt.nnonzeropixels,spokes.nspokes);  % pixel-dependent slice thickness (also spoke-dependent)
coords_3D_z=zeros(adj2.nx,adj2.ny,adj2.nz);
adj2.b0map=zeros(adj2.nx,adj2.ny,adj2.nz);
for i=1:spokes.nspokes
    mid_spoke_index=ceil( (spokes.sinc_time_and_spoke_to_time(1,i)+spokes.sinc_time_and_spoke_to_time(end,i))/2 );
    Gss_pola=sign( spokes.g(mid_spoke_index,3) );
    
    Z0(:,i)=b0./( Gss_pola*spokes.Gss-g );
    % Zeff(:,i)=spokes.sthick*spokes.Gss./( Gss_pola*spokes.Gss-g );    
    Zeff(:,i)=spokes.sthick;
    
    for j=1:opt.nnonzeropixels
        ii=opt.nonzeropixels(j,1);
        jj=opt.nonzeropixels(j,2);
        coords_3D_z(ii,jj,:)=([1:adj2.nz]-0.5)/adj2.nz*Zeff(j,i) - Zeff(j,i)/2 + Z0(j,i);
        
        % adj2.b0map(ii,jj,:)=adj.b0map(ii,jj) + g(j)*coords_3D_z(ii,jj,:)*gamma;
        adj2.b0map(ii,jj,:)=adj.b0map(ii,jj) + g(j)*( ([1:adj2.nz]'-0.5)/adj2.nz*adj2.lz - adj2.lz/2 )*gamma;
    end
    
    % these coordinates reflejct the fact that the slice thickness varies
    % on a pixel to picel basis
    opt2.coords_SS{i}=[ opt2.nonzeropixels(:,4:5) coords_3D_z(opt2.ind) ];
end




function xrf0=initialize_pulse(pulse_ini,nspokes,nchannels)

if exist(pulse_ini,'file')
    fprintf('\tUsing %s ...\n',pulse_ini);
    file=fopen(pulse_ini,'r');
    line=fgets(file);
    xrf0=[];
    for i=1:nchannels
        line=str2num( fgets(file) );
        if size(line,2)~=2*nspokes
            error('Initial pulse has wrong size.');
        end
        mag=line(1:2:end);
        pha=line(2:2:end)/180*pi;
        xrf0=[ xrf0;mag.*exp(1j*pha) ];
    end
    xrf0=reshape( xrf0,nspokes*nchannels,1 );
    xrf0=[real(xrf0);imag(xrf0)];
    fclose(file);
    
else
    xrf0=zeros( 2*nspokes*nchannels,1 );
    % xrf0=ones( 2*nspokes*nchannels,1 );
end



function opt=set_target_phase(opt,tar_pha)

if exist(tar_pha,'file')
    fprintf('\tUsing %s ...\n',tar_pha);
    
    pha=reshape(read_fdata(tar_pha,-1),opt.nx,opt.ny,opt.nz);
    opt.mttimg=abs(opt.mttimg) .* exp( 1j*pha );
    
    ind=opt.nonzeropixels(:,1) + (opt.nonzeropixels(:,2)-1)*size(opt.b1maps,2) ...
    + (opt.nonzeropixels(:,3)-1)*size(opt.b1maps,2)*size(opt.b1maps,3);  % could also use sub2ind()
    opt.mtt=opt.mttimg(ind);
end




























