

function h=dualhess_deltat(deltat,lambda,xrf,A_noRF_noTS,sinc_a,sinc_b_large,opt,spokes,sar,system,solver)
% function h=dualhess_deltat(deltat,lambda,xrf,A,opt,spokes,sar,solver)

[npix ncunk]=size(A_noRF_noTS);
nchannels=ncunk/spokes.nspokes;

spokes=time_shift_sinc_profiles(spokes,deltat*1e-6,system);

rf=xrf(1:ncunk) + 1j*xrf(ncunk+1:end);

%% HESSIAN OF MLS OBJECTIVE WRT TIME-SHIFTS
x=repmat( sinc_a*deltat',[npix 1] ) + sinc_b_large;
sincx=sinc(x/pi);
A_noRF=A_noRF_noTS.*sincx;
MT=A_noRF*rf;

y=abs(MT) - abs(opt.mtt);
z=sqrt( abs(opt.mtt)./abs(MT) ) .* exp(1j*angle(MT));
w=y./abs(MT);
z( abs(MT)==0 )=0;
w( abs(MT)==0 )=0;

A_noTS=A_noRF_noTS .* repmat(rf.',[npix 1]);

dx=sinc_a*( cos(x)./x-sin(x)./(x.^2) );
dx( find(abs(x)==0) )=0;
DM=A_noTS.*dx;

dx2=sinc_a^2*( sin(x).*(2-x.^2) - 2*x.*cos(x) )./(x.^3);
dx2( find(abs(x)==0) )=1/3;
D2M=A_noTS.*dx2;

% X_TS=DM'*diag(z);
X_TS=DM'.*repmat(z.',[ncunk 1]);  % faster

% Y_TS=DM'*diag(w)*DM;
Y_TS=( DM'.*repmat(w.',[ncunk 1]) )*DM;  % faster

h_OBJ=2*real(X_TS)*real(X_TS)' + 2*real(diag( D2M'*( y.*exp(1j*angle(MT)) ) )) + 2*real(Y_TS);

if solver==1  % Matlab solver
    LagMult=lambda.ineqnonlin;
else
    LagMult=lambda;
end


% HESSIAN OF SAR CONSTRAINTS WRT TIME-SHIFTS
nconst=sar.nvop+1;
if nconst>0
    subh=2*squeeze( sum( sar.sarmats .* repmat(LagMult(1:nconst),[1 nchannels nchannels]),1 ) );
    ch=kron( eye(spokes.nspokes),subh ).*(spokes.D2C);
    h_SAR=( real( (rf*rf').*ch ) - real(diag( (rf').*(rf.'*ch) )) )*1e-12;    
end

%% HESSIAN OF THE MAXIMUM TIME-SHIFT CONSTRAINTS
tmp=LagMult(nconst+1:nconst+ncunk);
% tmp=LagMult(1:ncunk);
h_MAX_TS=2*diag(tmp);

%% TOTAL HESSIAN
if nconst>0
    h=h_OBJ + h_SAR + h_MAX_TS;
else
    h=h_OBJ + h_MAX_TS;
end







