

function h=dualhess_jointRFDeltat(x,lambda,A_noRF_noTS,sinc_a,sinc_b_large,opt,spokes,sar,system,solver)
% function h=dualhess_MLS(x,lambda,A,opt,spokes,sar,solver)

[npix ncunk]=size(A_noRF_noTS);
nchannels=ncunk/spokes.nspokes;

deltat=x(1:ncunk);
spokes=time_shift_sinc_profiles(spokes,deltat*1e-6,system);

xrf=x(ncunk+1:end);
rf=xrf(1:ncunk) + 1j*xrf(ncunk+1:end);

%% HESSIAN OF MLS OBJECTIVE WRT RF
x=repmat( sinc_a*deltat',[npix 1] ) + sinc_b_large;
sincx=sinc(x/pi);
A_noRF=A_noRF_noTS.*sincx;
MT=A_noRF*rf;

y=abs(MT) - abs(opt.mtt);
z=sqrt( abs(opt.mtt)./abs(MT) ) .* exp(1j*angle(MT));
w=y./abs(MT);

z( abs(MT)==0 )=0;
w( abs(MT)==0 )=0;

% X_RF=A_noRF'*diag(z);
X_RF=A_noRF'.*repmat(z.',[ncunk 1]);  % faster

% Y_RF=A_noRF'*diag(w)*A_noRF;
Y_RF=( A_noRF'.*repmat(w.',[ncunk 1]) )*A_noRF;  % faster

h11=2*real(X_RF)*real(X_RF)' + 2*real(Y_RF);
h12=2*real(X_RF)*imag(X_RF)' - 2*imag(Y_RF);
h21=h12';
h22=2*imag(X_RF)*imag(X_RF)' + 2*real(Y_RF);
h_RF=[ h11 h12;h21 h22 ];

%% HESSIAN OF MLS OBJECTIVE WRT TIME-SHIFTS
A_noTS=A_noRF_noTS .* repmat(rf.',[npix 1]);

dx=sinc_a*( cos(x)./x-sin(x)./(x.^2) );
dx( find(abs(x)==0) )=0;
DM=A_noTS.*dx;

dx2=sinc_a^2*( sin(x).*(2-x.^2) - 2*x.*cos(x) )./(x.^3);
dx2( find(abs(x)==0) )=1/3;
D2M=A_noTS.*dx2;

% X_TS=DM'*diag(z);
X_TS=DM'.*repmat(z.',[ncunk 1]);  % faster

% Y_TS=DM'*diag(w)*DM;
Y_TS=( DM'.*repmat(w.',[ncunk 1]) )*DM;  % faster

h_TS=2*real(X_TS)*real(X_TS)' + 2*real(diag( D2M'*( y.*exp(1j*angle(MT)) ) )) + 2*real(Y_TS);

%% HESSIAN OF MLS OBJECTIVE WRT RF AND TIME-SHIFTS (CROSS TERMS)
U=A_noRF_noTS.*dx;

% Y_JOINT=DM'*diag(w)*A_noRF;
Y_JOINT=( DM'.*repmat(w.',[ncunk 1]) )*A_noRF;  % faster

D=diag(U'*(y.*exp(1j*angle(MT))));
h_RF_TS_real=2*real(X_TS)*real(X_RF)' + 2*real(D) + 2*real(Y_JOINT);
h_RF_TS_imag=2*real(X_TS)*imag(X_RF)' + 2*imag(D) - 2*imag(Y_JOINT);
h_RF_TS=[h_RF_TS_real h_RF_TS_imag];

%% TOTAL HESSIAN OF MLS OBJECTIVE
h_OBJ=[ h_TS     h_RF_TS;
    h_RF_TS' h_RF   ];

if solver==1  % Matlab solver
    LagMult=lambda.ineqnonlin;
else
    LagMult=lambda;
end

nconst=sar.nvop+1;
if nconst>0
    subh=2*squeeze( sum( sar.sarmats .* repmat(LagMult(1:nconst),[1 nchannels nchannels]),1 ) );
    
    % HESSIAN OF SAR CONSTRAINTS WRT TIME-SHIFTS
    ch=kron( eye(spokes.nspokes),subh ).*(spokes.D2C);
    h_SAR_TS=( real( (rf*rf').*ch ) - real(diag( (rf').*(rf.'*ch) )) )*1e-12;
    
    % HESSIAN OF SAR CONSTRAINTS WRT RF
    ch=kron( eye(spokes.nspokes),subh ).*(spokes.C);
    h_SAR_RF=[real(ch) -imag(ch)';imag(ch)' real(ch)];
    
    % HESSIAN OF SAR CONSTRAINTS WRT RF AND TIME-SHIFTS (CROSS TERMS)
    ch=kron( eye(spokes.nspokes),subh ).*(spokes.DC);
    TEMP=repmat(rf,[1 ncunk]).*ch + diag((rf.')*ch');
    h_SAR_RF_TS=[ real(TEMP) imag(TEMP) ]*1e-6;
    
    h_SAR=[ h_SAR_TS      h_SAR_RF_TS;
        h_SAR_RF_TS'  h_SAR_RF];
end

%% HESSIAN OF THE MAXIMUM TIME-SHIFT CONSTRAINTS
tmp=LagMult(nconst+1:nconst+ncunk);
% tmp=LagMult(1:ncunk);
h_MAX_TS=[2*diag(tmp) zeros(ncunk,2*ncunk);
    zeros(2*ncunk,ncunk) zeros(2*ncunk) ];


%% HESSIAN OF MAX POWER CONSTRAINTS
tmp=LagMult(nconst+ncunk+1:nconst+2*ncunk);
% tmp=LagMult(ncunk+1:2*ncunk);

h_MAX_RF=[zeros(ncunk) zeros(ncunk,2*ncunk);
    zeros(2*ncunk,ncunk) 2*diag([tmp;tmp]) ];


%% TOTAL HESSIAN
if nconst>0
    h=h_OBJ + h_SAR + h_MAX_TS + h_MAX_RF;
else
    h=h_OBJ + h_MAX_TS + h_MAX_RF;
end







