


function [deltat xrf f1 niters t2]=optimize_RF_and_time_shifts(deltat0,deltat_max,xrf0,A_noRF_noTS,sinc_a,sinc_b_large,spokes,opt,system,do_mls,sar,tol,disp)
% function [deltat xrf f1 niters t2]=optimize_RF_and_time_shifts(deltat0,deltat_max,xrf0,A_noRF_noTS,sinc_a,sinc_b_large,spokes,opt,system,do_mls,sar,tol,disp)

options = optimset('Largescale','off','Display','iter-detailed','MaxFunEvals',10^5,'TolFun',tol,'TolCon',tol,'MaxIter',5000,...
    'Algorithm','interior-point','GradObj','on','GradConstr','on','SubproblemAlgorithm','ldl-factorization');

if disp==1
    options.Display='iter-detailed';
else
    options.Display='none';
end

% make sure time-shifts are in range
deltat0( deltat0>deltat_max )=deltat_max;
deltat0( deltat0<-deltat_max )=-deltat_max;
spokes=time_shift_sinc_profiles(spokes,deltat0,system);

[npix ncunk]=size(A_noRF_noTS);
nchannels=ncunk/spokes.nspokes;

x=repmat( sinc_a*deltat0',[npix 1] ) + sinc_b_large;
sincx=sinc(x/pi);
A_noRF=A_noRF_noTS.*sincx;

% STEP #1: SOLVE MLS @ CONSTANT TIME-SHIFT
if disp>=0
    fprintf('Solving MLS problem @ constant time-shifts  ');
end
% test_grad( @(xrf)f0_MLS(xrf,A_noRF,opt),xrf0 );
% test_grad( @(xrf)fn(xrf,opt,spokes,sar,system,2),xrf0 );
% lambda=ones(1e5,1);
% test_hessian( @(xrf)f0_MLS(xrf,A_noRF,opt),@(xrf)dualhess_MLS(xrf,lambda,A_noRF,opt,spokes,sar,2),xrf0 );
% test_hessian( @(xrf)fn(xrf,opt,spokes,sar,system,2),@(xrf)dualhess_MLS(xrf,lambda,A_noRF,opt,spokes,sar,2),xrf0 );
options.Hessian='user-supplied';
options.HessFcn=@(xrf,lambda)dualhess_MLS(xrf,lambda,A_noRF,opt,spokes,sar,1);

tic;
[xrf0 f0_val exitflag output lambda]=fmincon( @(xrf)f0(xrf,A_noRF,opt,do_mls),xrf0,[],[],[],[],[],[],@(xrf)fn(xrf,opt,spokes,sar,system,1),options );
t1 = toc;
if disp>=0
    fprintf('(f0=%f, %f secs.)\n',f0_val,t1);
end

% STEP #2: SOLVE JOINT MLS PROBLEM
if disp>=0
    fprintf('Solving MLS problem over RF and time-shifts  ');
end
x0=[deltat0*1e6;xrf0];  % s -> us
sinc_a2=sinc_a*1e-6;
deltat_max2=deltat_max*1e6;
% test_grad( @(x)f0_jointRFDeltat(x,A_noRF_noTS,sinc_a2,sinc_b_large,spokes,opt,do_mls),x0 );
% test_grad( @(x)fn_jointRFDeltat(x,deltat_max2,opt,spokes,sar,system,2),x0 );
% lambda=ones(1e5,1);
% test_hessian( @(x)f0_jointRFDeltat(x,A_noRF_noTS,sinc_a2,sinc_b_large,spokes,opt,do_mls),@(x)dualhess_jointRFDeltat(x,lambda,A_noRF_noTS,sinc_a2,sinc_b_large,opt,spokes,sar,system,2),x0 );
% test_hessian( @(x)fn_jointRFDeltat(x,deltat_max2,opt,spokes,sar,system,2),@(x)dualhess_jointRFDeltat(x,lambda,A_noRF_noTS,sinc_a2,sinc_b_large,opt,spokes,sar,system,2),x0 );
options.Hessian='user-supplied';
options.HessFcn=@(x,lambda)dualhess_jointRFDeltat(x,lambda,A_noRF_noTS,sinc_a2,sinc_b_large,opt,spokes,sar,system,1);

tic;
[x f1 exitflag output lambda]=fmincon( @(x)f0_jointRFDeltat(x,A_noRF_noTS,sinc_a2,sinc_b_large,spokes,opt,1),x0, ...
    [],[],[],[],[],[],@(x)fn_jointRFDeltat(x,deltat_max2,opt,spokes,sar,system,1),options );
t2=toc;
niters=output.iterations;

if disp>=0
    fprintf('(f0=%f, %f secs.)\n',f1,t2);
end

deltat=x(1:ncunk)*1e-6;
xrf=x(ncunk+1:end);

[f df MT]=f0_jointRFDeltat(x,A_noRF_noTS,sinc_a,sinc_b_large,spokes,opt,1);

% % STEP #3: REFINE RF
% if disp>=0
%     fprintf('Solving MLS problem @ constant time-shifts  ');
% end
% 
% x=repmat( sinc_a*deltat',[npix 1] ) + sinc_b_large;
% sincx=sinc(x/pi);
% A_noRF=A_noRF_noTS.*sincx;
% 
% options.Hessian='user-supplied';
% options.HessFcn=@(xrf,lambda)dualhess(xrf,lambda,A_noRF,opt,spokes,sar,1,do_mls);    
% [xrf f0_val exitflag output lambda]=fmincon( @(xrf)f0(xrf,A_noRF,opt,do_mls),xrf0,[],[],[],[],[],[],@(xrf)fn(xrf,opt,spokes,sar,system,1),options );
% if disp>=0
%     fprintf('(f0=%f)\n',f0_val);
% end











