

function spokes2=prepare_pulse(spokes,system,display,deltat,deltat_max)
% function spokes2=prepare_pulse(spokes,system,display,deltat,deltat_max)

gamma=42.576*1e6;

% compute gradient and k-space
spokes2=prepare_gradient(spokes,system,deltat_max);

N=spokes2.ntimesperspoke;
g=spokes2.g(1:N,:)*spokes2.sdir(:);
spokes2.Gss=abs(g(ceil(N/2),1));  % slice selection plateau
omega=gamma*(spokes2.Gss)*(spokes2.sthick);

times=([1:N]'-0.5-N/2)*system.deltat;
ind_rf=find(spokes2.mask_rf(1:N)>0);
Nrf=ind_rf(end)-ind_rf(1);
Trf=Nrf*system.deltat - 2*spokes2.deltat_max;

spokes2.sinc=zeros(N,system.ncoils,spokes.nspokes);
spokes2.Dsinc=zeros(N,system.ncoils,spokes.nspokes);
for i=1:system.ncoils
    for j=1:spokes.nspokes
        x=omega*(times-deltat(i+(j-1)*system.ncoils));
        
        f1=sinc(x);
        f2=rect( (times-deltat(i+(j-1)*system.ncoils))/Trf );    
        f3=0.54 + 0.46*cos( 2*pi*(times-deltat(i+(j-1)*system.ncoils))/Trf );  % hanning window
        
        df1=pi*omega*( cos(pi*x)./(pi*x)-sin(pi*x)./((pi*x).^2) );
        df1( find(abs(x)==0) )=0;
        
        df3=-2*pi*0.46/Trf*sin( 2*pi*(times-deltat(i+(j-1)*system.ncoils))/Trf );
        
        spokes2.sinc(:,i,j)=f1.*f2.*f3;    
        spokes2.Dsinc(:,i,j)=df1.*f2.*f3 + f1.*f2.*df3;    
    end
end

% correlation matrix
spokes2.sinc_corr_mat=cell(spokes.nspokes);
spokes2.Dsinc_corr_mat=cell(spokes.nspokes);
spokes2.C=zeros(system.ncoils*spokes.nspokes);
spokes2.DC=zeros(system.ncoils*spokes.nspokes);
for i=1:spokes.nspokes
    spokes2.sinc_corr_mat{i}=spokes2.sinc(:,:,i)'*spokes2.sinc(:,:,i) / spokes2.ntimes;
    spokes2.Dsinc_corr_mat{i}=spokes2.sinc(:,:,i)'*spokes2.Dsinc(:,:,i) / spokes2.ntimes;
    
    ind=[1:system.ncoils]+(i-1)*system.ncoils;
    spokes2.C(ind,ind)=spokes2.sinc_corr_mat{i};
    spokes2.DC(ind,ind)=spokes2.Dsinc_corr_mat{i};    
end



if strcmpi(display,'on')
    figure;
    times=[1:spokes2.ntimes]'*system.deltat;
    subplot(2,1,1); plot(times*1e3,spokes2.g(:,1)*1e3,'Color','blue','LineWidth',2); hold on;
    plot(times*1e3,spokes2.g(:,2)*1e3,'Color','red','LineWidth',2);
    plot(times*1e3,spokes2.g(:,3)*1e3,'Color','green','LineWidth',2);
    legend('Gx','Gy','Gz','Location','EastOutside'); legend('boxoff');
    title('Gradient trajectory [mT/m]');

    subplot(2,1,2); plot(times*1e3,spokes2.k(:,1),'Color','blue','LineWidth',2); hold on;
    plot(times*1e3,spokes2.k(:,2),'Color','red','LineWidth',2);
    plot(times*1e3,spokes2.k(:,3),'Color','green','LineWidth',2);
    legend('kx','ky','kz','Location','EastOutside'); legend('boxoff');
    title('K-space trajectory [1/m]'); xlabel('Time [ms]');

    for i=1:spokes.nspokes
        f=figure;
        plot(times(1:N,1)*1e3,spokes2.sinc(:,:,i),'LineWidth',2);
        title(sprintf('Spoke %d sub-pulses [a.u.]',i)); xlabel('Time [ms]');
    end
end




